# Pulled from / tweaked -> https://github.com/jmhale/okta-awscli/blob/master/oktaawscli/aws_auth.py

""" Handles auth to Okta and returns SAML assertion """
import base64
# pylint: disable=C0325,R0912,C1801
import sys
from collections import namedtuple

import requests
from bs4 import BeautifulSoup as bs

from figcli.models.sso.okta.okta_auth import OktaAuth
from figcli.utils.utils import *

log = logging.getLogger(__name__)


class Okta:
    """ Handles auth to Okta and returns SAML assertion """

    def __init__(self, okta_auth: OktaAuth):
        self.app_link = okta_auth.app_link
        self.auth = okta_auth
        self.https_base_url = f"https://{okta_auth.base_url}"

    def get_apps(self, session_id):
        """ Gets apps for the user """
        sid = "sid=%s" % session_id
        headers = {'Cookie': sid}
        resp = requests.get(
            self.https_base_url + '/api/v1/users/me/appLinks',
            headers=headers).json()
        aws_apps = []
        for app in resp:
            if app['appName'] == "amazon_aws":
                aws_apps.append(app)
        if not aws_apps:
            log.error("No AWS apps are available for your user. \
                Exiting.")
            sys.exit(1)

        aws_apps = sorted(aws_apps, key=lambda app: app['sortOrder'])
        app_choice = 0 if len(aws_apps) == 1 else None
        if app_choice is None:
            print("Available apps:")
            for index, app in enumerate(aws_apps):
                app_name = app['label']
                print("%d: %s" % (index + 1, app_name))

            app_choice = int(input('Please select AWS app: ')) - 1
        log.info("Selected app: %s" % aws_apps[app_choice]['label'])
        return aws_apps[app_choice]['label'], aws_apps[app_choice]['linkUrl']

    def get_saml_assertion(self, html):
        """ Returns the SAML assertion from HTML """
        soup = bs(html.text, "html.parser")
        assertion = ''

        for input_tag in soup.find_all('input'):
            if input_tag.get('name') == 'SAMLResponse':
                assertion = input_tag.get('value')

        if not assertion:
            log.error("SAML assertion not valid: " + assertion)

        return assertion

    def get_assertion(self):
        """ Main method to get SAML assertion from Okta """
        session = self.auth.get_session()

        if not self.app_link:
            print("dafuq??")
            # app_name, app_link = self.get_apps(session_id)
        else:
            app_name = None
            app_link = self.app_link
        sid = "sid=%s" % session.session_id
        headers = {'Cookie': sid}
        resp = requests.get(app_link, headers=headers)
        assertion = self.get_saml_assertion(resp)
        if len(assertion) < 20:
            raise InvalidSessionError("Invalid assertion returned from OKTA")

        return assertion

    @staticmethod
    def extract_available_roles_from(assertion):
        aws_attribute_role = 'https://aws.amazon.com/SAML/Attributes/Role'
        attribute_value_urn = '{urn:oasis:names:tc:SAML:2.0:assertion}AttributeValue'
        roles = []
        role_tuple = namedtuple("RoleTuple", ["principal_arn", "role_arn"])
        root = ET.fromstring(base64.b64decode(assertion))
        for saml2attribute in root.iter('{urn:oasis:names:tc:SAML:2.0:assertion}Attribute'):
            if saml2attribute.get('Name') == aws_attribute_role:
                for saml2attributevalue in saml2attribute.iter(attribute_value_urn):
                    roles.append(role_tuple(*saml2attributevalue.text.split(',')))
        return roles



from dataclasses import dataclass
from sty import fg, FgRegister


@dataclass
class Palette:
    # Default pallete colors
    BLUE = (52, 213, 235)
    GREEN = (51, 222, 136)
    RED = (240, 70, 87)
    YELLOW = (240, 240, 70)

    def __init__(self, blue=BLUE, green=GREEN, red=RED, yellow=YELLOW):
        self.fg_bl: FgRegister = fg(*blue)
        self.fg_gr: FgRegister = fg(*green)
        self.fg_rd: FgRegister = fg(*red)
        self.fg_yl: FgRegister = fg(*yellow)

        # Prompt Toolkit HEX colors
        self.bl: str = '#%02x%02x%02x' % blue
        self.gr: str = '#%02x%02x%02x' % green
        self.rd: str = '#%02x%02x%02x' % red
        self.yl: str = '#%02x%02x%02x' % yellow

        self.rs = fg.rs


import sys

import pexpect
from figcli.test.cli.config import *
from figcli.test.cli.figgy import FiggyTest
from figcli.config import *
from figcli.utils.utils import *
from figcli.test.cli.dev.put import DevPut
from figcli.test.cli.dev.delete import DevDelete
import time

class DevList(FiggyTest):

    def __init__(self, extra_args=""):
        super().__init__(None, extra_args=extra_args)

    def run(self):
        # self.successful_list()
        self.test_empty_input()

    def test_empty_input(self):
        self.step("Testing empty input for list.")
        self._setup(1, 3)
        print(f"Testing {CLI_NAME} config {Utils.get_first(list_com)} --env dev")
        print("Waiting for cache population.")
        time.sleep(80)
        child = pexpect.spawn(f'{CLI_NAME} config {Utils.get_first(list_com)} --env {DEFAULT_ENV} '
                              f'--skip-upgrade {self.extra_args}', timeout=10, encoding='utf-8')
        child.expect('.*Please input a namespace prefix.*')
        child.sendline("")
        child.expect('.*Please input a namespace prefix.*')
        child.sendline("")
        child.expect('.*Please input a namespace prefix.*')
        child.sendline(dump_prefix)
        child.expect(f'.*1.*{dump_prefix}.*2.*{dump_prefix}.*3.*{param_1}.*Selection.*')
        child.sendline('3')
        child.expect(f'.*Value.*{param_1_val}.*')
        child.sendline('')
        child.sendline('3')
        child.sendline('')
        print("List successful. Cleaning up.")

        self._cleanup(1, 3)

    def successful_list(self):
        self._setup(1, 3)
        self.step("Testing successful list.")
        print(f"Testing {CLI_NAME} config {Utils.get_first(list_com)} --env dev")
        child = pexpect.spawn(f'{CLI_NAME} config {Utils.get_first(list_com)} --env {DEFAULT_ENV} '
                              f'--skip-upgrade {self.extra_args}',
                              timeout=10, encoding='utf-8')

        print("Waiting for cache population.")
        time.sleep(80)
        child.expect('.*Please input a namespace prefix.*')
        child.sendline(dump_prefix)
        child.expect(f'.*1.*{dump_prefix}.*2.*{dump_prefix}.*3.*{param_1}.*Selection.*')
        child.sendline('3')
        child.expect(f'.*Value.*{param_1_val}.*')
        child.sendline('')
        child.sendline('3')
        child.sendline('')
        print("List successful. Cleaning up.")

        self._cleanup(1, 3)

    def _setup(self, min: int, max: int):
        put = DevPut(extra_args=self.extra_args)
        put.add(param_1, param_1_val, param_1_desc, add_more=True)
        for i in range(min, max):
            more = i < max - 1
            put.add_another(f'{param_1}-{i}', param_1_val, f'{param_1_desc}-{i}', add_more=more)


    def _cleanup(self, min: int, max: int):
        delete = DevDelete(extra_args=self.extra_args)
        delete.delete(param_1, delete_another=True)
        for i in range(min, max):
            delete.delete(f'{param_1}-{i}', delete_another=i < max - 1)




from dataclasses import dataclass
from typing import List, Optional

from figcli.models.defaults.provider import Provider
from figcli.models.role import Role
from figcli.models.run_env import RunEnv
from tabulate import tabulate


@dataclass
class AssumableRole:
    account_id: int
    run_env: RunEnv
    role: Optional[Role]
    provider_name: Optional[str]
    profile: Optional[str]

    def tabulate_data(self) -> List[str]:
        return [f'{self.account_id[0:5]} REDACTED', self.run_env.env, self.role.role]

    def tabulate_header(self):
        return ["Account #", "Environment", "Role"]

    def print(self):
        print(self.__dict__)

    @staticmethod
    def from_profile(profile: str):
        return AssumableRole(
            account_id=1234567899,
            run_env=RunEnv(env=profile),
            profile=profile,
            role=Role(profile),
            provider_name=Provider.PROFILE.value)

    @property
    def role_arn(self) -> str:
        return f"arn:aws:iam::{self.account_id}:role/{self.role.full_name}"

    def __str__(self):
        return tabulate(
            [self.tabulate_data()],
            headers=self.tabulate_header(),
            tablefmt="grid",
            numalign="center",
            stralign="left",
        )

    def __eq__(self, other):
        return self.account_id == other.account_id and \
               self.role == other.role and \
               self.run_env == other.run_env and \
               self.profile == other.profile

    def __hash__(self):
        return hash(f'{self.account_id}-{self.role}-{self.run_env}')


import sys

import pexpect

from figcli.test.cli.config import *
from figcli.test.cli.dev.delete import DevDelete
from figcli.test.cli.dev.sync import DevSync
from figcli.test.cli.figgy import FiggyTest
from figcli.utils.utils import *


class DevValidate(FiggyTest):

    def __init__(self, extra_args=""):
        super().__init__(None, extra_args=extra_args)
        self.missing_key = '/app/ci-test/v1/config12'

    def run(self):
        self.step("Testing successful validate.")
        self.validate_success()
        self.step("Testing validate with errors")
        self.validate_error()

    def validate_success(self):
        sync = DevSync(extra_args=self.extra_args)
        sync.sync_success()

        print(f"Testing: {CLI_NAME} config {Utils.get_first(validate)} --env {DEFAULT_ENV} "
              f"--config figcli/test/assets/success/figgy.json {self.extra_args}")
        child = pexpect.spawn(f'{CLI_NAME} config {Utils.get_first(validate)} --env {DEFAULT_ENV} '
                              f'--config figcli/test/assets/success/figgy.json --skip-upgrade {self.extra_args}',
                              encoding='utf-8', timeout=10)
        child.logfile = sys.stdout
        child.expect(".*Success.*")
        print("VALIDATE SUCCESS VERIFIED")

    def validate_error(self):
        delete = DevDelete(extra_args=self.extra_args)
        delete.delete(self.missing_key)
        print(f"Testing: {CLI_NAME} config {Utils.get_first(validate)} --env {DEFAULT_ENV} "
              f"--config figcli/test/assets/error/figgy.json {self.extra_args}")
        child = pexpect.spawn(f'{CLI_NAME} config {Utils.get_first(validate)} --env {DEFAULT_ENV} '
                              f'--config figcli/test/assets/error/figgy.json --skip-upgrade {self.extra_args}', timeout=10)
        child.expect('.*missing at least one.*')
        print("VALIDATE ERROR VERIFIED")
